# @summary creates a script for overclocking the gpus.
#
# @param user
#   The user to assign ownership over the settings.
#
# @param user_id
#   The id of the user to run the overclock script (requires sudo access).
#
# @param gpu_freq
#   Controls the GPUGraphicsClockOffset from nvidia-settings for each GPU.
#
# @param power_limit
#   Specifies maximum power limit in watts for each GPU.
#
# @param fan_speed
#   The percentage to set the speed of the fan.
#
# @param vram_freq
#   Controls the GPUMemoryTransferRateOffset from nvidia-settings for each GPU.
#   Note, GUI tools generally double the value you see when it is set via command line, so you may need to double this setting 
#   for the desired result.
#
# @param gpus
#   The number of GPUs to control in the system.
#
# @param enable_service
#   Add overclock as a service to ensure overclock is called on boot.
#
class nvidia::overclock (
  String  $user,
  Integer $user_id,
  Integer $gpu_freq,
  Integer $vram_freq,
  Integer $power_limit,
  Integer $fan_speed,
  Integer $gpus           = size($facts['gpus_pci_ids']),
  Boolean $enable_service = false,
) {
  file { '/usr/local/bin/overclock.bash':
    ensure  => file,
    owner   => $user,
    group   => $user,
    mode    => '0755',
    content => epp('nvidia/overclock.bash',{
        user_id     => $user_id,
        gpu_freq    => $gpu_freq,
        vram_freq   => $vram_freq,
        power_limit => $power_limit,
        fan_speed   => $fan_speed,
        gpus        => $gpus,
    }),
  }
  if $enable_service {
    systemd::unit_file { 'overclock.service':
      content   => epp('nvidia/overclock.service.epp',
        {
          user => $user,
        }
      ),
      subscribe => File['/usr/local/bin/overclock.bash'],
    }

    service { 'overclock':
      ensure    => running,
      subscribe => Systemd::Unit_file['overclock.service'],
    }
  }
}
